#! /usr/bin/env python
# zettel - a todo list and appointment calender
# Copyright (C) 2003  Jochen Voss <voss@mathematik.uni-kl.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path
# FIX PATH

import locale
from kasten import Kasten, verzettelt
from query import query_new_card
from version import VERSION

######################################################################
## auxiliary functions

def show_commands(fd):
    global commands
    fd.write('\n')
    fd.write('available commands:\n')
    for s in commands:
        if not s.startswith("fn_"): continue
        doc=eval(s+".__doc__")
        if doc:
            doc=doc.splitlines()[0]
        else:
            doc=s.replace("fn_","",1)
        fd.write("    "+doc+"\n")
        
def usage(fd):
    fd.write('usage: zettel command [args]\n')
    show_commands(fd)

def show_error(msg):
    sys.stderr.write('zettel: error: '+msg+'\n')
    sys.stderr.write('\n')
    usage(sys.stderr)
    sys.exit(1)

######################################################################
## user-visible commands

def fn_help(box,cmd=None):
    """help [CMD]          explain commands and exit

Explain CMD if specified, or emit a list of all available commands
to stdout."""
    if cmd:
        try:
            fn=eval("fn_"+cmd)
        except:
            show_error('unknown command "'+cmd+'"')
        else:
            sys.stdout.write(fn.__doc__+"\n")
    else:
        usage(sys.stdout)
        sys.stdout.write('\n')
        sys.stdout.write('Report bugs to <voss@mathematik.uni-kl.de>.\n')

def fn_version(box):
    """version             show version information"""
    print "Zettel "+VERSION
    print "Copyright (C) 2003  Jochen Voss"
    print """Zettel comes with NO WARRANTY, to the extent permitted by law.
You may redistribute copies of GNU Emacs under the terms of the
GNU General Public License.  For more information about these
matters, see the files named COPYING."""

def fn_list(box):
    """list                list all entries

List all items from the database.  The format is as follows: first
comes the id of the entry, then a "*" if the entry's activation time
has passed, then a letter to indicate the state of the entry
(A=active, S=suspended, D=done), and finally the title.

To get the full information about an entry from the list use the
"show" command."""
    box.list_all()

def fn_current(box,limit=0):
    """current [LIMIT]     list current entries

List current entries from the database.  If the optional argument
limit is given, restrict the output to LIMIT (randomly chosen) lines.

This command is meant to be used from scripts.  It emits the short
description of each current entry to stdout.  The format of the output
lines is the same as with the "list" command."""
    try:
        l=int(limit)
    except ValueError, e:
        show_error("invalid number of lines (%s)"%limit)
    box.list_current(l)

def fn_show(box,id):
    """show ID             show one entry

Show all information associated to entry ID.  Use the "list" command
to find out the lists of valid ID values."""
    box.show_by_id(id)

def fn_add(box):
    """add                 add a new entry

Spawn an editor to enter a new item and then add this item to
the database."""
    zdict=query_new_card()
    box.add(zdict)

def fn_remove(box,id):
    """remove ID           permanently remove an entry

Permanently remove the item ID from the database.  Caution:
there are no further questions and there is no undo functionality.
So check the ID carefully."""
    box.remove(id)

def fn_done(box,id):
    """done ID             mark an entry as done

Mark entry ID as done, but leave it in the database."""
    box.set_state(id, "done")

def fn_suspend(box,id):
    """suspend ID          mark an entry as suspended

Mark entry ID as suspend.  The entry stays in the database, but it is
not displayed with the "active" command any more.  Use the "resume" command
to reactivate the entry."""
    box.set_state(id, "suspended")

def fn_resume(box,id):
    """resume ID           reactivate an entry

Reactive an entry, which was previously marked as suspended or done."""
    box.set_state(id, "active")

def fn_defer(box,id,n):
    """defer ID N          defer entry ID for N days

Change the activation time of entry ID to make it inactive for now.
It will become active again in N days."""
    try:
        days=float(n)*24*3600
    except ValueError:
        show_error("invalid number of days (%s)"%n)
    box.defer(id,days)

def fn_priority(box,id,val):
    """priority ID VAL     set the priority value of an entry

Positive numbers indicate high priorities, negative values low
priorities.  High priority entries are displayed more prominently."""
    try:
        p=float(val)
    except ValueError:
        show_error("invalid priority value")
    box.set_priority(id,p)

######################################################################
## main program

locale.setlocale(locale.LC_ALL, "")

zettelkasten=os.path.expanduser("~/.zettelkasten")

commands=dir()
if len(sys.argv)<2: show_error("missing command")
cmd=sys.argv[1]
try:
    fn=eval("fn_"+cmd)
except:
    show_error('unknown command "'+cmd+'"')

box=Kasten(zettelkasten)

try:
    fn(box,*sys.argv[2:])
except verzettelt, err:
    show_error(err.msg)
except TypeError, err:
    if str(err).startswith("fn_"):
        show_error('wrong number of arguments')
    else:
        raise err

box.shut()
